#!/bin/sh
#----------------------------------------------------------------------------------------
# /var/install/config.d/chroot.sh - configuration generator script for chroot
#
# Copyright (c) 2008-2015 The Eisfair Team, team(at)eisfair(dot)org
#
# Creation:    2008-07-06  jed
# Last Update: $Id: chroot.sh 37960 2015-05-17 17:47:05Z jed $
#
# Parameters:  chroot [removeall]
#                - removeall - remove all user specific modifications from /etc/passwd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------------------
# This script is based on a script created by Wolfgang Fuschlberger,
# see http://www.fuschlberger.net/programs/ssh-scp-sftp-chroot-jail/
# for further information.
#----------------------------------------------------------------------------------------

# include eislib
. /var/install/include/eislib
. /var/install/include/check-eisfair-version
. /var/install/include/jedlib

#exec 2> /tmp/chroot-trace-$$.test
#set -x
#testroot=/tmp/jedchroot                                             # only for testing
 testroot=""

pgmname=`basename $0`

# set platform specific parameters
case ${EISFAIR_SYSTEM} in
    eisfair-1)
        # eisfair-1
        chroot_configpath=${testroot}/var/chroot
        generate_sudoers=${testroot}/etc/sudoers.d/chroot
        ;;
    *)
        # default to eisfair-2
        chroot_configpath=${testroot}/data/packages/chroot
        generate_sudoers=${testroot}/etc/sudoers.chroot
        ;;
esac

### set file names and pathes ###
ssh_configpath=${testroot}/etc/ssh

chrootfile=${testroot}/etc/config.d/chroot
sshd_configfile=${ssh_configpath}/sshd_config
packagefile=${testroot}/var/install/packages/chroot

generate_saveuser="${chroot_configpath}/chroot-save-userinfo"
generate_savepath="${chroot_configpath}/chroot-save-pathinfo"

default_custom_filelist="${chroot_configpath}/custom-default-filelist.txt"
custom_filelist=''       # custom-<username>-filelist.txt will be set later

chroot_apps="/bin/bash /bin/cp /usr/bin/dircolors /bin/ls /bin/mkdir /bin/mv /bin/rm /bin/rmdir /bin/sh /bin/su /usr/bin/groups /usr/bin/id /sbin/unix_chkpwd"
chroot_ssh_apps="/usr/bin/ssh /usr/bin/scp"
chroot_rsync_apps="/usr/bin/rsync"

# other parameters
chroot_version="v`grep "<version>" ${packagefile} | sed 's#<version>\(.*\)</version>#\1#'`"
default_shell='/bin/bash'
dest_label='jail'

### load configuration ###
. ${chrootfile}
chmod 600 ${chrootfile}

#----------------------------------------------------------------------------
# create directories in jaildir
# input : $1 - jail directory
#----------------------------------------------------------------------------
create_chroot_dirs()
{
    jdir="${1}"

    mecho "- checking directories in ${dest_label} directory ..."

    # create directories in jail that do not exist yet
    for DIR in /bin /dev /etc /etc/pam.d /home /sbin /tmp /usr /usr/bin /usr/lib
    do
        if [ ! -d "${jdir}${DIR}" ]
        then
            mecho "  + creating directory ${jdir}${DIR} ..."
            create_dirs_with_perms ${DIR} ${jdir}
        fi
    done
}

#----------------------------------------------------------------------------
# create chroot devices in jaildir
# input : $1 - jail directory
#----------------------------------------------------------------------------
create_chroot_devs()
{
    jdir="${1}"

    mecho "- creating devices in ${dest_label} directory ..."

    # create necessary devices
    if [ ! -r ${jdir}/dev/null ]
    then
        mecho "  + creating device /dev/null ..."
        mknod -m 666 ${jdir}/dev/null c 1 3
    fi

    if [ ! -r ${jdir}/dev/random ]
    then
        mecho "  + creating device /dev/random ..."
        mknod -m 644 ${jdir}/dev/random c 1 8
    fi

    if [ ! -r ${jdir}/dev/tty ]
    then
        mecho "  + creating device /dev/tty ..."
        mknod -m 666 ${jdir}/dev/tty c 5 0
    fi

    if [ ! -r ${jdir}/dev/urandom ]
    then
        mecho "  + creating device /dev/urandom ..."
        mknod ${jdir}/dev/urandom c 1 9
    fi

    if [ ! -r ${jdir}/dev/zero ]
    then
        mecho "  + creating device /dev/zero ..."
        mknod -m 666 ${jdir}/dev/zero c 1 5
    fi
}

#----------------------------------------------------------------------------
# create files in in jaildir
# input : $1 - jail directory
#----------------------------------------------------------------------------
create_chroot_files()
{
    jdir="${1}"

    # create /usr/bin/groups in the jail
    {
        echo "#!/bin/bash"
        echo "id -Gn"
    } > ${jdir}/usr/bin/groups

    chmod 755 ${jdir}/usr/bin/groups

    # add root user to etc/passwd
    if [ ! -f ${jdir}/etc/passwd ]
    then
        grep "^root" /etc/passwd > ${jdir}/etc/passwd
    fi

    # add root users to etc/shadow
    if [ ! -f ${jdir}/etc/shadow ]
    then
        grep "^root" /etc/shadow > ${jdir}/etc/shadow
    fi

    if [ ! -f ${jdir}/etc/group ]
    then
        {
            grep "^root" /etc/group

            # add the group for all users to etc/group (otherwise there is
            # a nasty error message and probably because of that changing
            # directories doesn't work with winSCP)

            grep "^users" /etc/group

        } > ${jdir}/etc/group
    fi
}

#----------------------------------------------------------------------------
# create directories and set permissions/access rights
# input : $1 - source directory path
#         $2 - jail directory
#----------------------------------------------------------------------------
create_dirs_with_perms()
{
    spath=$1
    jpath=$2

    # make sure that the jail directory exists
    if [ ! -d ${jpath} ]
    then
        mkdir -p ${jpath}
    fi

    # split path into pieces
    _oldifs=$IFS
    IFS=/

    set - ${spath}

    IFS=$_oldifs

    # check if sub-directories exist
    spath=''
    dpath="${jpath}"
    while [ $# -gt 0 ]
    do
        if [ "${1}" != "" ]
        then
            spath=${spath}/${1}
            dpath=${dpath}/${1}

            if [ ! -d ${dpath} ]
            then
                # create sub-directory
                mkdir ${dpath}
            fi

            # set access permissions
            saccess="`get_accessrights ${spath} | cut -d: -f1`"
            sowner="`get_accessrights ${spath}  | cut -d: -f2`"
            sgroup="`get_accessrights ${spath}  | cut -d: -f3`"

            chmod ${saccess} ${dpath}
            chown ${sowner}  ${dpath}
            chgrp ${sgroup}  ${dpath}
        fi

        shift
    done
}

#----------------------------------------------------------------------------
# check if file exists in chroot and if it's the same file
# input : $1 - file one
#         $2 - file two
# output: 0 - do not copy
#         1 - copy file
#----------------------------------------------------------------------------
check_chroot_file ()
{
    sfile=$1
    dfile=$2

    if [ -f ${sfile} ]
    then
        if [ ! -f ${dfile} ]
        then
            # file doesn't exist ...
            # print one dot per file
            mecho --info -n "."

            cp -p ${sfile} ${dfile}
        else
            # file doesn't exist ...
            # compare file ...
            diff "${sfile}" "${dfile}" > /dev/null

            if [ $? -ne 0 ]
            then
                # files differ ...
                # print one dot per file
                mecho --info -n "."

                cp -p ${sfile} ${dfile}
            fi
        fi
    fi
}

#----------------------------------------------------------------------------
# copy files to jaildir
# input : $1 - jail directory
#----------------------------------------------------------------------------
copy_chroot_files()
{
    jdir="${1}"

    # Copy the apps and the related libs
    mecho -n "- copying necessary library files to ${dest_label} ..."

    # create temporary files with mktemp, if that doesn't work for some
    # reason use the old method with $HOME/ldlist[2] (so I don't have to
    # check the existence of the mktemp package / binary at the beginning
    tmpfile1=/tmp/chroot-ldd-1.$$
    tmpfile2=/tmp/chroot-ldd-2.$$

    for app in ${apps}
    do
        # first of all, check that this application exists
        if [ -f ${app} ]
        then
            # check that the directory exists; create it if not.
            app_path=`dirname ${app}`

            if [ ! -d "${jdir}${app_path}" ]
            then
                create_dirs_with_perms ${app_path} ${jdir}
            fi

            # If the files in the chroot are on the same file system as
            # the original files you should be able to use hard links
            # instead of copying the files, too. Symbolic links cannot
            # be used, because the original files are outside the chroot.

            check_chroot_file "${app}" "${jdir}${app}"

            # get list of necessary libraries
            ldd ${app} >> ${tmpfile1}
        else
            echo "  + ${app} not found!"
        fi
    done

    mecho

    if [ -s ${tmpfile1} ]
    then
        for libs in `cat ${tmpfile1}`
        do
            frst_char="`echo ${libs} | cut -c1`"

            if [ "${frst_char}" = "/" ]
            then
                echo "${libs}" >> ${tmpfile2}
            fi
        done
    fi

    # copy depending libraries files
    mecho -n "- copying depending library files to ${dest_label} ..."

    if [ -s ${tmpfile2} ]
    then
        for lib in `cat ${tmpfile2}`
        do
            create_dirs_with_perms `dirname ${lib}` ${jdir}

            # If the files in the chroot are on the same file system as
            # the original files you should be able to use hard links
            # instead of copying the files, too. Symbolic links cannot
            # be used, because the original files are outside the chroot.

            check_chroot_file "${lib}" "${jdir}${lib}"
        done
    fi

    mecho

    # Now, cleanup the 2 files we created for the library list
    rm -f ${tmpfile1}
    rm -f ${tmpfile2}

    # Necessary files that are not listed by ldd.
    mecho -n "- copying additional library files to ${dest_label} ..."

    for lib in /lib/libnss_compat.so.2 /lib/libnsl.so.1 /lib/libnss_files.so.2 /lib/libnss_dns.so.2 /lib/libcap.so.1
    do
        check_chroot_file "${lib}" "${jdir}${lib}"
    done

    mecho

    # The following lines in, perhaps it works then (motto: if you
    # can't find the needed library just copy all of them)
  # cp /lib/*.* ${jdir}/lib/
  # cp /lib/lib64/*.* ${jdir}/lib/lib64/

    # if you are using PAM you need stuff from /etc/pam.d/ in the jail
    mecho "- copying pam files to jail ..."
    cp -p /etc/pam.d/* ${jdir}/etc/pam.d/

    # ... and of course the PAM-modules...
    cp -p -r /lib/security ${jdir}/lib/

    # ... and something else useful for PAM
    cp -p -r /etc/security ${jdir}/etc/
    cp -p /etc/login.defs ${jdir}/etc/

    for fname in /etc/DIR_COLORS /etc/resolv.conf /etc/nsswitch.conf /etc/aliases.std /etc/inittab.std /etc/services.std
    do
        check_chroot_file "${fname}" "${jdir}${fname}"
    done

    # Don't give more permissions than necessary
    chown root.root ${jdir}/bin/su
    chmod 700       ${jdir}/bin/su
}

#----------------------------------------------------------------------------
# copy files to jaildir
# input : $1 - jail directory
#         $2 - username
#----------------------------------------------------------------------------
copy_custom_chroot_files()
{
    jdir="${1}"
    usr="${2}"

    # Copy the apps and the related libs
    mecho -n "- copying files from '${default_custom_filelist}' to ${dest_label} ..."

    if [ -s ${default_custom_filelist} ]
    then
        # check access of default custom file list
        faccess1="`get_accessrights ${default_custom_filelist} | cut -d: -f1 | cut -c1`"
        faccess2="`get_accessrights ${default_custom_filelist} | cut -d: -f1 | cut -c2`"
        faccess3="`get_accessrights ${default_custom_filelist} | cut -d: -f1 | cut -c3-4`"
        fowner="`get_accessrights ${default_custom_filelist} | cut -d: -f2`"
        fgroup="`get_accessrights ${default_custom_filelist} | cut -d: -f3`"

        errflag=0
        if [ "${faccess1}" != "0" -o "${faccess3}" != "00" ]
        then
            errflag=1
            mecho
            mecho --warn "  + Ignoring file list because access rights '${faccess1}${faccess2}${faccess3}' are not properly set!"
        fi

        if [ "${fowner}" != "root" -o "${fgroup}" != "root" ]
        then
            errflag=1
            mecho
            mecho --warn "  + Ignoring file list because ownership '${fowner}:${fgroup}' are not properly set!"
        fi

        if [ ${errflag} -eq 0 ]
        then
            # access rights are ok
            cat ${default_custom_filelist} | grep -v "^#" | \
            while read line
            do
                app=`trim_spaces "${line}"`

                if [ "${app}" != "" ]
                then
                    # first of all, check that this application exists
                    if [ -f ${app} ]
                    then
                        # check that the directory exists; create it if not.
                        app_path=`dirname ${app}`

                        if [ ! -d "${jdir}${app_path}" ]
                        then
                            create_dirs_with_perms ${app_path} ${jdir}
                        fi

                        # If the files in the chroot are on the same file system as
                        # the original files you should be able to use hard links
                        # instead of copying the files, too. Symbolic links cannot
                        # be used, because the original files are outside the chroot.

                        check_chroot_file "${app}" "${jdir}${app}"
                    else
                        mecho "  + ${app} not found!"
                    fi
                fi
            done

            mecho
        fi
    else
        # error
        mecho
        mecho "  + Default custom file list doesn't exist."
    fi

    # Copy the apps and the related libs
    custom_filelist="${chroot_configpath}/custom-${usr}-filelist.txt"
    mecho -n "- copying files from '${custom_filelist}' to ${dest_label} ..."

    if [ -f ${custom_filelist} ]
    then
        # check access of default custom file list
        faccess1="`get_accessrights ${custom_filelist} | cut -d: -f1 | cut -c1`"
        faccess2="`get_accessrights ${custom_filelist} | cut -d: -f1 | cut -c2`"
        faccess3="`get_accessrights ${custom_filelist} | cut -d: -f1 | cut -c3-4`"
        fowner="`get_accessrights ${custom_filelist} | cut -d: -f2`"
        fgroup="`get_accessrights ${custom_filelist} | cut -d: -f3`"

        errflag=0
        if [ "${faccess1}" != "0" -o "${faccess3}" != "00" ]
        then
            errflag=1
            mecho
            mecho --warn "  + Ignoring file list because access rights '${faccess1}${faccess2}${faccess3}' are not properly set!"
        fi

        if [ "${fowner}" != "root" -o "${fgroup}" != "root" ]
        then
            errflag=1
            mecho
            mecho --warn "  + Ignoring file list because ownership '${fowner}:${fgroup}' are not properly set!"
        fi

        if [ ${errflag} -eq 0 ]
        then
            # access rights are ok
            cat ${custom_filelist} | grep -v "^#" | \
            while read line
            do
                app=`trim_spaces "${line}"`

                if [ "${app}" != "" ]
                then
                    cpflag=0

                    # first of all, check that this application exists
                    if [ -f ${app} ]
                    then
                        # check that the directory exists; create it if not.
                        app_path=`dirname ${app}`

                        if [ ! -d "${jdir}${app_path}" ]
                        then
                            create_dirs_with_perms ${app_path} ${jdir}
                        fi

                        # If the files in the chroot are on the same file system as
                        # the original files you should be able to use hard links
                        # instead of copying the files, too. Symbolic links cannot
                        # be used, because the original files are outside the chroot.

                        check_chroot_file "${app}" "${jdir}${app}"
                    else
                        echo "  + ${app} not found!"
                    fi
                fi
            done

            mecho
        fi
    else
        # error
        mecho
        mecho "  + custom user file list doesn't exist."
    fi
}

#----------------------------------------------------------------------------
# update sudoers file
#----------------------------------------------------------------------------
update_sodoers()
{
    # modifiy /etc/sudoersd/chroot to enable chroot-ing for users
    echo "modifying ${generate_sudoers} ..."

    if [ ${CHROOT_N} -gt 0 ]
    then
        #-------------------------------------------------------------------------
        print_short_header "${generate_sudoers}" "${pgmname}" "chroot" "${chroot_version}" > ${generate_sudoers}
        #-------------------------------------------------------------------------

        jdx=1
        while [ ${jdx} -le ${CHROOT_N} ]
        do
            eval act='$CHROOT_'${jdx}_'ACTIVE'
            eval usr='$CHROOT_'${jdx}'_USER'

            if [ "${act}" = "yes" ]
            then
                # active account, go on ...
                # frank ALL=NOPASSWD: /usr/bin/chroot, /bin/su - frank
                echo "${usr} ALL=NOPASSWD: `which chroot`, /bin/su - ${usr}" >> ${generate_sudoers}
            fi

            jdx=`expr ${jdx} + 1`
        done
    else
        # no users defined, remove file
        rm -f ${generate_sudoers}
    fi

    chmod 0440 ${generate_sudoers}

    if [ "${EISFAIR_SYSTEM}" != "eisfair-1" ]
    then
        # run update command on non-eisfair-1 systems
        /var/install/bin/update-sudoers ${pgmname}
    fi
}

#----------------------------------------------------------------------------
# create accounts in jaildir
# input : $1 - jail directory
#         $2 - username
#         $3 - home directory
#         $4 - shell script
#----------------------------------------------------------------------------
create_chroot_accounts()
{
    jdir="${1}"
    usr="${2}"
    hdir="${3}"
    sscript="${4}"

    echo "- creating chroot accounts in ${dest_label} ..."

    jdx=1
    while [ ${jdx} -le ${CHROOT_N} ]
    do
        eval act='$CHROOT_'${jdx}_'ACTIVE'
        eval usr='$CHROOT_'${jdx}'_USER'

        if [ "${act}" = "yes" ]
        then
            # active account, go on ...
            # if the system uses one account/one group we write the
            # account's group to etc/group

            grps=`id -Gn "${usr}"`

            if [ -f ${jdir}/etc/group ]
            then
                # add all groups of a user
                for GRP in ${grps}
                do
                    grep -q "^${GRP}:" ${jdir}/etc/group

                    if [ $? -ne 0 ]
                    then
                        grep "^${GRP}:" /etc/group >> ${jdir}/etc/group
                    fi
                done
            fi

            # grep the username which was given to us from /etc/passwd and
            # add it to ./etc/passwd replacing the $HOME with the directory
            # as it will then appear in the jail

            if [ -f ${jdir}/etc/passwd ]
            then
                grep -q "^${usr}:" ${jdir}/etc/passwd

                if [ $? -ne 0 ]
                then
                    grep "^${usr}:" /etc/passwd | sed -e "s#${jdir}##" -e "s#${sscript}#/bin/bash#" >> ${jdir}/etc/passwd
                fi
            fi

            # write the user's line from /etc/shadow to /home/jail/etc/shadow
            if [ -f ${jdir}/etc/shadow ]
            then
                grep -v "^${usr}:" ${jdir}/etc/shadow > ${jdir}/etc/shadow.tmp
                mv ${jdir}/etc/shadow.tmp ${jdir}/etc/shadow
            fi

            grep "^${usr}:" /etc/shadow >> ${jdir}/etc/shadow

            chmod 600 ${jdir}/etc/shadow
        fi

        jdx=`expr ${jdx} + 1`
    done
}

#----------------------------------------------------------------------------
# remove user from jaildir
# input : $1 - jail directory
#         $2 - username
#----------------------------------------------------------------------------
remove_chroot_account()
{
    jdir="$1"
    usr="$2"

    if [ -f ${jdir}/etc/passwd ]
    then
        grep -q "^${usr}" ${jdir}/etc/passwd

        if [ $? -eq 0 ]
        then
            grep -v "^${usr}" ${jdir}/etc/passwd > ${jdir}/etc/passwd.tmp
            mv ${jdir}/etc/passwd.tmp ${jdir}/etc/passwd
        fi

        grep -q "^${usr}" ${jdir}/etc/shadow

        if [ $? -eq 0 ]
        then
            grep -v "^${usr}" ${jdir}/etc/shadow > ${jdir}/etc/shadow.tmp
            mv ${jdir}/etc/shadow.tmp ${jdir}/etc/shadow

            chmod 600 ${jdir}/etc/shadow
        fi
    fi
}

#----------------------------------------------------------------------------
# create profile file in chroot jail
# input : $1 - jail directory
#----------------------------------------------------------------------------
create_chroot_profile()
{
    jdir="${1}"

    if [ ! -f ${jdir}/etc/profile ]
    then
        echo "- creating /etc/profile ..."

        {
            echo '#----------------------------------------------------------------------------------------'
            echo '# /etc/profile - global profile'
            echo '#'
            echo '# Copyright (c) 2008 The Eisfair Team, c/o Frank Meyer, frank(at)eisfair(dot)org'
            echo '#'
            echo '# Creation:    2008-07-08 jed'
            echo '# Last Update: 2008-07-08 jed'
            echo '#'
            echo '# This program is free software; you can redistribute it and/or modify'
            echo '# it under the terms of the GNU General Public License as published by'
            echo '# the Free Software Foundation; either version 2 of the License, or'
            echo '# (at your option) any later version.'
            echo '#----------------------------------------------------------------------------------------'
            echo
            echo 'umask 022'
            echo 'cd ${HOME}            # for ash: prevent call of /bin/pwd'
            echo
            echo 'if [ "$TERM" != "xterm" ]'
            echo 'then'
            echo "    TERM='linux'"
            echo 'fi'
            echo 'export TERM'
            echo
            echo 'PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin'
            echo 'export PATH'
            echo
            echo 'if [ "${LOGNAME}" = "root" ]'
            echo 'then'
            echo "    PS1=\"`hostname` # \""
            echo 'else'
            echo "    PS1=\"`hostname` > \""
            echo '    PATH=$PATH:.'
            echo 'fi'
            echo
            echo 'export PS1'
            echo
            echo 'LANG="C"'
            echo 'export LANG'
        } > ${jdir}/etc/profile
    fi
}

#----------------------------------------------------------------------------
# save user information
# input: $1 - jail directory
#        $2 - username
#----------------------------------------------------------------------------
save_user_details()
{
    jdir="$1"
    usr="$2"

    echo "- saving user details ..."

    fflag=1

    if [ -f ${generate_saveuser} ]
    then
        # check if entry already exist
        grep -q "^${usr}:" ${generate_saveuser}
        fflag=$?
    fi

    if [ ${fflag} -eq 1 ]
    then
        grep -q "^${usr}:" /etc/passwd

        if [ $? -eq 0 ]
        then
            # save entry from passwd file
            echo "`grep "^${usr}:" /etc/passwd`:${jdir}" >> ${generate_saveuser}
        else
            # user:x:uid:gid:description:homedir:shell:jaildir
            echo "${usr}:::jaildir-only-user:::${jdir}" >> ${generate_saveuser}
        fi

        chmod 600 ${generate_saveuser}
    fi
}

#----------------------------------------------------------------------------
# save path information
# input: $1 - jailpath
#----------------------------------------------------------------------------
save_path_details()
{
    jdir="$1"

    echo "- saving home-dir/jail path '${jdir}' ..."

    fflag=1

    if [ -f ${generate_savepath} ]
    then
        # check if entry already exist
        grep -q "^${jdir}:" ${generate_savepath}
        fflag=$?
    fi

    if [ ${fflag} -eq 1 ]
    then
        # not found, add it ...
        echo "${jdir}:" >> ${generate_savepath}

        chmod 600 ${generate_savepath}
    fi
}

#----------------------------------------------------------------------------
# clean path information
#----------------------------------------------------------------------------
clean_path_details()
{
    mecho -n "cleaning ${dest_label} path list ..."

    if [ -f ${generate_savepath} ]
    then
        while read line
        do
            mecho --info -n "."

            delflag=1

            sdir=`echo "${line}" | cut -d: -f1`

            jdx=1
            while [ ${jdx} -le ${CHROOT_N} ]
            do
                eval jdir='$CHROOT_'${jdx}'_JAILPATH'

                if [ "${jdir}" = "" ]
                then
                    # use default path
                    jdir="${CHROOT_DEFAULT_JAILPATH}"
                fi

                if [ "${sdir}" = "${jdir}" ]
                then
                    # path found, don't delete
                    delflag=0
                fi

                jdx=`expr ${jdx} + 1`
            done

            if [ ${delflag} -eq 1 ]
            then
                if [ -d ${sdir} ]
                then
                # echo "deleting ${sdir} ..."
                    rm -fr ${sdir}
                fi
            else
                # don't delete entry, save it ...
                echo "${sdir}:" >> ${generate_savepath}.tmp
            fi
        done < ${generate_savepath}

        # update jailpath list
        if [ -f ${generate_savepath}.tmp ]
        then
            mecho --info -n "."

            mv ${generate_savepath}.tmp ${generate_savepath}
        fi
    else
        # no users defined
        mecho "nothing to do."
    fi

    mecho
}

#----------------------------------------------------------------------------
# restore account information
# input: $1 - action -> 'removeall'
#----------------------------------------------------------------------------
restore_user_details()
{
    action="${1}"

    tmpfile1=/tmp/chroot-restore-1.$$
    tmpfile2=/tmp/chroot-restore-2.$$

    if [ -f ${generate_saveuser} ]
    then
        cp ${generate_saveuser} ${tmpfile1}

        while read line
        do
            delflag=1

            susr=`echo "${line}" | cut -d: -f1`        # get system user name
            jdir=`echo "${line}" | cut -d: -f8`        # get current jail directory

            if [ "${action}" != "removeall" -a "${START_CHROOT}" = "yes" ]
            then
                jdx=1
                while [ ${jdx} -le ${CHROOT_N} ]
                do
                    eval act='$CHROOT_'${jdx}'_ACTIVE'
                    eval usr='$CHROOT_'${jdx}'_USER'

                    if [ "${susr}" = "${usr}" -a "${act}" = "yes" ]
                    then
                        # user found, don't delete
                        delflag=0
                    fi

                    jdx=`expr ${jdx} + 1`
                done
            fi

            if [ ${delflag} -eq 1 ]
            then
                # make sure the account exists
                grep -q "^${susr}:" /etc/passwd

                if [ $? -eq 0 ]
                then
                    # make sure the save file exists
                    if [ -f ${generate_saveuser} ]
                    then
                        # restore home directory of system user
                        hdir=`grep "^${susr}:" ${generate_saveuser} | cut -d: -f6`

                        if [ "${hdir}" = "" ]
                        then
                            # delete jail-only user
                            mecho "removing jail-only user '${susr}' ..."
                            /var/install/bin/remove-user "${susr}" "no"
                        else
                            /var/install/bin/modify-user -d "${susr}" "${hdir}"

                            # restore shell of system user
                            shell=`grep "^${susr}:" ${generate_saveuser} | cut -d: -f7 | sed "s#^*chroot-shell-*#${default_shell}#"`

                            if [ "${shell}" != "" ]
                            then
                                /var/install/bin/modify-user -s "${susr}" "${shell}"
                            fi
                        fi

                        # remove jail-shell-script
                        rm -f /bin/chroot-shell-${susr}
                    fi

                    # remove account from chroot jail
                    remove_chroot_account ${jdir} ${susr}
                fi

                # remove user from list
                grep -v "^${susr}:" ${tmpfile1} > ${tmpfile2}
                mv ${tmpfile2} ${tmpfile1}
            fi
        done < ${generate_saveuser}

        # update jail-user list
        if [ -f ${tmpfile1} ]
        then
            mv ${tmpfile1} ${generate_saveuser}
        fi
    else
        # no users defined
        mecho "nothing to do."
    fi
}

#========================================================================================
# main
#========================================================================================

if [ -z "${PATH}" ]
then
    PATH=/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin
fi

case "$1" in
    'removeall')
        restore_user_details 'removeall'
        ;;
    *)
        if [ "${START_CHROOT}" = "yes" ]
        then
            user=`whoami`

            if is_root ${user}
            then
                mecho
                mecho "version: ${chroot_version}"

                mecho "checking required files ..."
                for FNAME in which chroot sudo dirname awk
                do
                    type ${FNAME} > /dev/null 2> /dev/null

                    if [ $? -ne 0 ]
                    then
                        echo "- ${FNAME} program not found!"
                        exit 1
                    fi
                done

                mecho "checking sftp-server ..."
                if [ -f ${sshd_configfile} ]
                then
                    if !(grep -v "^#" ${sshd_configfile} | grep -i -q sftp-server)
                    then
                        mecho --warn "- no sftp-server is running on this system!"
                    else
                        sftp_server=$(grep -v "^#" ${sshd_configfile} | grep -i sftp-server | awk  '{ print $3}')

                        chroot_ssh_apps="${chroot_ssh_apps} ${sftp_server}"
                    fi
                else
                    mecho "- ${sshd_configfile} not found!"
                    exit 1
                fi

                if [ ${CHROOT_N} -gt 0 ]
                then
                    idx=1
                    while [ ${idx} -le ${CHROOT_N} ]
                    do
                        eval active='$CHROOT_'${idx}'_ACTIVE'
                        eval user='$CHROOT_'${idx}'_USER'

                        mecho "processing chroot (${idx}) ..."
                        mecho "- checking user '${user}' ..."

                        if [ "${active}" = "yes" ]
                        then
                            # active account, go on ...
                            eval name='$CHROOT_'${idx}'_COMMENT'
                            eval jailpath='$CHROOT_'${idx}'_JAILPATH'
                            eval lockuser='$CHROOT_'${idx}'_LOCK_USER'
                            eval sshapps='$CHROOT_'${idx}'_ENABLE_SSH_APPS'
                            eval rsyncapps='$CHROOT_'${idx}'_ENABLE_RSYNC_APPS'

                            if [ "${jailpath}" = "" ]
                            then
                                # used default jail path
                                jailpath="${CHROOT_DEFAULT_JAILPATH}"
                            fi

                            # check ssh parameter
                            if [ "${sshapps}" = "" ]
                            then
                                sshapps="${CHROOT_DEFAULT_ENABLE_SSH_APPS}"
                            fi

                            apps="${chroot_apps}"

                            if [ "${sshapps}" = "yes" ]
                            then
                                apps="${apps} ${chroot_ssh_apps}"
                            else
                                # remove files
                                for app in ${chroot_ssh_apps}
                                do
                                    if [ -f ${jailpath}${app} ]
                                    then
                                        rm -f ${jailpath}${app}
                                    fi
                                done
                            fi

                            # check rsync parameter
                            if [ "${rsyncapps}" = "" ]
                            then
                                rsyncapps="${CHROOT_DEFAULT_ENABLE_RSYNC_APPS}"
                            fi

                            if [ "${rsyncapps}" = "yes" ]
                            then
                                apps="${apps} ${chroot_rsync_apps}"
                            else
                                # remove files
                                for app in ${chroot_rsync_apps}
                                do
                                    if [ -f ${jailpath}${app} ]
                                    then
                                        rm -f ${jailpath}${app}
                                    fi
                                done
                            fi

                            save_user_details ${jailpath} ${user}
                            save_path_details ${jailpath}

                            # create/modify system user
                            group=''
                            uid=''
                            gid=''
                            homedir="/home/${user}"
                            shell="${default_shell}"
                            jailhomedir="${jailpath}/home/${user}"
                            jailshell="/bin/chroot-shell-${user}"
                            pass='!'

                            # creating jail shell script
                            echo "- creating ${jailshell} ..."
                            {
                                echo '#!/bin/sh'
                                #-------------------------------------------------------------------------
                                print_short_header "${shell}" "${pgmname}" "chroot" "${chroot_version}"
                                #-------------------------------------------------------------------------
                                echo "`which sudo` `which chroot` ${jailpath} /bin/su - \$USER" \"\$@\"
                            } > ${jailshell}

                            chmod 755 ${jailshell}

                            # create home of home directory before adding user
                            if [ ! -d "${jailpath}/home" ]
                            then
                                mkdir -p "${jailpath}/home"
                            fi

                            # check if account exists
                            grep -q "^${user}:" /etc/passwd

                            if [ $? -ne 0 ]
                            then
                                # user doesn't exist
                                mecho "- adding user '${user}' ..."

                                if [ "${lockuser}" = 'yes' ]
                                then
                                    # lock user in jail
                                    /var/install/bin/add-user -l ${user} "" "${uid}" "${gid}" "${name}" "${jailhomedir}" "${jailshell}"
                                else
                                    # don't lock user in jail
                                    /var/install/bin/add-user -l ${user} "" "${uid}" "${gid}" "${name}" "${homedir}" "${shell}"
                                fi
                            fi

                            # user exists, make sure that the homedir and shell has properly set
                            hname=`grep "^${user}:" /etc/passwd|cut -d: -f6`
                            sname=`grep "^${user}:" /etc/passwd|cut -d: -f7`

                            if [ "${lockuser}" = 'yes' ]
                            then
                                # lock user in jail
                                dest_label='jail'

                                if [ "${hname}" != "${jailhomedir}" ]
                                then
                                    mecho "  + modifying home directory '${hname}' -> '${jailhomedir}' ..."
                                    /var/install/bin/modify-user -d "${user}" "${jailhomedir}"
                                fi

                                if [ "${sname}" != "${jailshell}" ]
                                then
                                    mecho "  + modifying shell '${sname}' -> '${jailshell}' ..."

                                    # don't use standard script because we don't want to modify /etc/shells
                                  # /var/install/bin/modify-user -s "${user}" "${shell}"
                                    /usr/sbin/usermod -s "${jailshell}" "${user}"
                                fi
                            else
                                # don't lock user in jail
                                dest_label='home-dir'

                                if [ "${hname}" != "${homedir}" ]
                                then
                                    mecho "  + modifying home directory '${hname}' -> '${homedir}' ..."
                                    /var/install/bin/modify-user -d "${user}" "${homedir}"
                                fi

                                if [ "${sname}" != "${shell}" ]
                                then
                                    mecho "  + modifying shell '${sname}' -> '${shell}' ..."

                                    # don't use standard script because we don't want to modify /etc/shells
                                  # /var/install/bin/modify-user -s "${user}" "${shell}"
                                    /usr/sbin/usermod -s "${shell}" "${user}"
                                fi
                            fi

                            create_chroot_dirs  ${jailpath}
                            create_chroot_devs  ${jailpath}
                            create_chroot_files ${jailpath}
                            copy_chroot_files   ${jailpath}
                            copy_custom_chroot_files ${jailpath} ${user}

                            create_chroot_profile ${jailpath}
                            create_chroot_accounts ${jailpath} ${user} ${homedir} ${shell}

                            # create home directory and set access rights within jail
                            mkdir -p       ${jailhomedir}
                            chown ${user}: ${jailhomedir}
                            chmod 700      ${jailhomedir}
                        else
                            mecho --warn "- skipping user because it has been deactivated!"
                        fi

                        idx=`expr ${idx} + 1`
                    done

                    update_sodoers
                else
                    mecho --error "At least one user needs to be jailed!"
                fi
                restore_user_details
                clean_path_details

                mecho "finished."
            else
                mecho --error "You must be root to run this script!"
                exit 1
            fi
        else
            update_sodoers

            restore_user_details
            clean_path_details
        fi
        ;;
esac

#========================================================================================
# end
#========================================================================================
